@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -16,5 +16,11 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
+
+#ifndef FILENAME_H
+#define FILENAME_H
  
-gchar* filename_parse(gchar *in, RS_PHOTO *photo);
+extern gchar* filename_parse(gchar *in, RS_PHOTO *photo);
+extern GtkWidget *rs_filename_chooser_button_new(gchar **filename, const gchar *conf_key);
+
+#endif
