@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -19,25 +19,28 @@
 
 #include <glib.h>
 #include <gtk/gtk.h>
-#include "color.h"
-#include "matrix.h"
-#include "rs-batch.h"
 #include "rawstudio.h"
 #include "conf_interface.h"
 #include "gtk-interface.h"
 #include "filename.h"
+#include "gtk-helper.h"
+#include "rs-cms.h"
 #include <gettext.h>
 #include <lcms.h>
 
-void gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
-void gui_cms_di_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
-void gui_cms_ex_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
-void gui_cms_intent_combobox_changed(GtkComboBox *combobox, gpointer user_data);
-void gui_cms_in_profile_button_clicked(GtkButton *button, gpointer user_data);
-void gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_data);
-void gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data);
+static void gui_confbox_changed(GtkComboBox *filetype_combo, gpointer callback_data);
+static gboolean gui_confbox_deleted(GtkWidget *widget, GdkEvent *event, gpointer callback_data);
+static void gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
+static void gui_cms_di_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
+static void gui_cms_ex_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data);
+static void gui_cms_intent_combobox_changed(GtkComboBox *combobox, gpointer user_data);
+static void gui_cms_in_profile_button_clicked(GtkButton *button, gpointer user_data);
+static void gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_data);
+static void gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data);
+static void spinner_set_conf(GtkSpinButton *spinner, gpointer user_data);
+static void slider_with_spinner_set_conf(GtkSpinButton *spinner, gpointer user_data);
 
-gchar *color_profiles[] = {
+static const gchar *color_profiles[] = {
 	"*.icc", 
 	"*.icm", 
 	"*.ICC", 
@@ -45,6 +48,201 @@ gchar *color_profiles[] = {
 	NULL
 };
 
+enum {
+	COMBO_CONF_ID = 0,
+	COMBO_TEXT,
+	COMBO_PTR,
+	COMBO_ROWS,
+};
+
+static void
+gui_confbox_changed(GtkComboBox *combo, gpointer callback_data)
+{
+	RS_CONFBOX *confbox = (RS_CONFBOX *) callback_data;
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	gchar *conf_id;
+	gpointer ptr;
+
+	gtk_combo_box_get_active_iter(GTK_COMBO_BOX(confbox->widget), &iter);
+	model = gtk_combo_box_get_model(GTK_COMBO_BOX(confbox->widget));
+	gtk_tree_model_get(model, &iter,
+		COMBO_CONF_ID, &conf_id,
+		COMBO_PTR, &ptr,
+		-1);
+	rs_conf_set_string(confbox->conf_key, conf_id);
+
+	if (confbox->callback)
+		confbox->callback(ptr, confbox->user_data);
+	return;
+}
+
+static gboolean
+gui_confbox_deleted(GtkWidget *widget, GdkEvent *event, gpointer callback_data)
+{
+	RS_CONFBOX *confbox = (RS_CONFBOX *) callback_data;
+	gui_confbox_destroy(confbox);
+
+	return(TRUE);
+}
+
+gpointer
+gui_confbox_get_active(RS_CONFBOX *confbox)
+{
+	GtkTreeIter iter;
+	GtkTreeModel *model;
+	gpointer ptr;
+
+	if(gtk_combo_box_get_active_iter(GTK_COMBO_BOX(confbox->widget), &iter))
+	{
+		model = gtk_combo_box_get_model(GTK_COMBO_BOX(confbox->widget));
+		gtk_tree_model_get(model, &iter,
+			COMBO_PTR, &ptr,
+			-1);
+		return(ptr);
+	}
+	else
+		return(NULL);
+}
+
+void
+gui_confbox_add_entry(RS_CONFBOX *confbox, const gchar *conf_id, const gchar *text, gpointer *user_data)
+{
+	GtkTreeIter iter;
+	gtk_list_store_append (confbox->model, &iter);
+	gtk_list_store_set (confbox->model, &iter,
+		COMBO_CONF_ID, conf_id,
+		COMBO_TEXT, text, 
+		COMBO_PTR, user_data,
+		-1);
+
+	return;
+}
+
+void
+gui_confbox_load_conf(RS_CONFBOX *confbox, gchar *default_value)
+{
+	gchar *conf_id;
+	gchar *value;
+	gchar *needle = default_value;
+	GtkTreeModel *model;
+	GtkTreePath *path;
+	GtkTreeIter iter;
+
+	value = rs_conf_get_string(confbox->conf_key);
+	if (value)
+		needle = value;
+
+	model = gtk_combo_box_get_model(GTK_COMBO_BOX(confbox->widget));
+	path = gtk_tree_path_new_first();
+	gtk_tree_model_get_iter(model, &iter, path);
+	gtk_tree_path_free(path);
+
+	do {
+		gtk_tree_model_get(model, &iter, COMBO_CONF_ID, &conf_id, -1);
+		if (g_str_equal(conf_id, needle))
+		{
+			gtk_combo_box_set_active_iter(GTK_COMBO_BOX(confbox->widget), &iter);
+			break;
+		}
+	} while(gtk_tree_model_iter_next (model, &iter));
+
+	if (value)
+		g_free(value);
+	return;
+}
+
+void
+gui_confbox_set_callback(RS_CONFBOX *confbox, gpointer user_data, void (*callback)(gpointer active, gpointer user_data))
+{
+	confbox->user_data = user_data;
+	confbox->callback = callback;
+	return;
+}
+
+RS_CONFBOX *
+gui_confbox_new(const gchar *conf_key)
+{
+	RS_CONFBOX *confbox;
+	GtkCellRenderer *renderer;
+
+	confbox = g_new(RS_CONFBOX, 1);
+
+	confbox->widget = gtk_combo_box_new();
+	confbox->model = gtk_list_store_new(COMBO_ROWS, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_POINTER);
+	gtk_combo_box_set_model (GTK_COMBO_BOX(confbox->widget), GTK_TREE_MODEL (confbox->model));
+	renderer = gtk_cell_renderer_text_new ();
+	gtk_cell_layout_pack_start (GTK_CELL_LAYOUT (confbox->widget), renderer, TRUE);
+	gtk_cell_layout_set_attributes (GTK_CELL_LAYOUT (confbox->widget), renderer,
+		"text", COMBO_TEXT, NULL);
+	confbox->conf_key = conf_key;
+	g_signal_connect ((gpointer) confbox->widget, "changed", G_CALLBACK (gui_confbox_changed), confbox);
+	g_signal_connect ((gpointer) confbox->widget, "delete_event", G_CALLBACK(gui_confbox_deleted), confbox);
+	confbox->user_data = NULL;
+	confbox->callback = NULL;
+
+	return(confbox);
+}
+
+void
+gui_confbox_destroy(RS_CONFBOX *confbox)
+{
+	gtk_widget_destroy(confbox->widget);
+	g_free(confbox);
+
+	return;
+}
+
+GtkWidget *
+gui_confbox_get_widget(RS_CONFBOX *confbox)
+{
+	return(confbox->widget);
+}
+
+RS_CONFBOX *
+gui_confbox_filetype_new(const gchar *conf_key)
+{
+	extern RS_FILETYPE *filetypes;
+	RS_FILETYPE *filetype = filetypes;
+	RS_CONFBOX *confbox;
+
+	confbox = gui_confbox_new(conf_key);
+
+	while(filetype)
+	{
+		if (filetype->save)
+		{
+			gui_confbox_add_entry(confbox, filetype->id, filetype->description, (gpointer) filetype);
+		}
+		filetype = filetype->next;
+	}
+	gui_confbox_load_conf(confbox, "jpeg");
+
+	return(confbox);
+}
+
+void
+checkbox_set_conf(GtkToggleButton *togglebutton, gpointer user_data)
+{
+	const gchar *path = user_data;
+	rs_conf_set_boolean(path, togglebutton->active);
+	return;
+}
+
+GtkWidget *
+checkbox_from_conf(const gchar *conf, gchar *label, gboolean default_value)
+{
+	gboolean check = default_value;
+	GtkWidget *checkbox;
+	rs_conf_get_boolean(conf, &check);
+	checkbox = gtk_check_button_new_with_label(label);
+	gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(checkbox),
+		check);
+	g_signal_connect ((gpointer) checkbox, "toggled",
+		G_CALLBACK (checkbox_set_conf), (gpointer) conf);
+	return(checkbox);
+}
+
 GtkWidget *gui_tooltip_no_window(GtkWidget *widget, gchar *tip_tip, gchar *tip_private)
 {
 	GtkWidget *e;
@@ -78,16 +276,15 @@ gui_batch_directory_entry_changed(GtkEntry *entry, gpointer user_data)
 }
 
 void
-gui_batch_filename_entry_changed(GtkEntry *entry, gpointer user_data)
+gui_batch_filename_entry_changed(GtkComboBox *combobox, gpointer user_data)
 {
-	rs_conf_set_string(CONF_BATCH_FILENAME, gtk_entry_get_text(entry));
+	rs_conf_set_string(CONF_BATCH_FILENAME, gtk_combo_box_get_active_text(combobox));
 	return;
 }
 
 void
-gui_batch_filetype_entry_changed(GtkEntry *entry, gpointer user_data)
+gui_batch_filetype_combobox_changed(gpointer active, gpointer user_data)
 {
-	rs_conf_set_string(CONF_BATCH_FILETYPE, gtk_entry_get_text(entry));
 	return;
 }
 
@@ -97,12 +294,12 @@ gui_export_changed_helper(GtkLabel *label)
 	gchar *parsed = NULL;
 	gchar *directory;
 	gchar *filename;
-	gchar *filetype;
 	GString *final;
+	RS_FILETYPE *filetype;
 
 	directory = rs_conf_get_string(CONF_EXPORT_DIRECTORY);
 	filename = rs_conf_get_string(CONF_EXPORT_FILENAME);
-	filetype = rs_conf_get_string(CONF_EXPORT_FILETYPE);
+	rs_conf_get_filetype(CONF_EXPORT_FILETYPE, &filetype);
 
 	parsed = filename_parse(filename, NULL);
 
@@ -115,8 +312,7 @@ gui_export_changed_helper(GtkLabel *label)
 	g_string_append(final, parsed);
 	g_free(parsed);
 	g_string_append(final, ".");
-	g_string_append(final, filetype);
-	g_free(filetype);
+	g_string_append(final, filetype->ext);
 	g_string_append(final, "</small>");
 
 	gtk_label_set_markup(label, final->str);
@@ -148,117 +344,69 @@ gui_export_filename_entry_changed(GtkComboBox *combobox, gpointer user_data)
 	return;
 }
 
-void
+static void
 cms_enable_toggled(GtkToggleButton *togglebutton, gpointer user_data)
 {
 	RS_BLOB *rs = (RS_BLOB *) user_data;
-	rs_local_cachedir(togglebutton->active);
 	rs_conf_set_boolean(CONF_CMS_ENABLED, togglebutton->active);
-	rs->cms_enabled = togglebutton->active;
-	rs_cms_prepare_transforms(rs);
-	update_preview_callback(NULL, rs);
+	rs_cms_enable(rs->cms, togglebutton->active);
+	update_previewtable_callback(NULL, rs);
 	return;
 }
 
-void
+static void
 gui_cms_in_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 {
 	RS_BLOB *rs = (RS_BLOB *) user_data;
 	gchar *filename;
-
 	rs_conf_set_integer(CONF_CMS_IN_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
-	filename = rs_get_profile(RS_CMS_PROFILE_IN);
-
-	if (rs->loadProfile)
-	{
-		cmsCloseProfile(rs->loadProfile);
-		rs->loadProfile = NULL;
-	}
-
-	if (filename)
-	{
-		rs->loadProfile = cmsOpenProfileFromFile(filename, "r");
-		g_free(filename);
-	}
-
-	rs_cms_prepare_transforms(rs);
+	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_IN);
+	rs_cms_set_profile(rs->cms, PROFILE_INPUT, filename);
+	g_free(filename);
 	update_preview_callback(NULL, rs);
 	return;
 }
 
-void
+static void
 gui_cms_di_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 {
 	RS_BLOB *rs = (RS_BLOB *) user_data;
 	gchar *filename;
 
 	rs_conf_set_integer(CONF_CMS_DI_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
-	filename = rs_get_profile(RS_CMS_PROFILE_DISPLAY);
-
-	if (rs->displayProfile)
-	{
-		cmsCloseProfile(rs->displayProfile);
-		rs->displayProfile = NULL;
-	}
-
-	if (filename)
-	{
-		rs->displayProfile = cmsOpenProfileFromFile(filename, "r");
-		g_free(filename);
-	}
-
-	rs_cms_prepare_transforms(rs);
+	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_DISPLAY);
+	rs_cms_set_profile(rs->cms, PROFILE_DISPLAY, filename);
+	g_free(filename);
 	update_preview_callback(NULL, rs);
 	return;
 }
 
-void
+static void
 gui_cms_ex_profile_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 {
 	RS_BLOB *rs = (RS_BLOB *) user_data;
 	gchar *filename;
 
 	rs_conf_set_integer(CONF_CMS_EX_PROFILE_SELECTED, gtk_combo_box_get_active(GTK_COMBO_BOX(combobox)));
-	filename = rs_get_profile(RS_CMS_PROFILE_EXPORT);
-
-	if (rs->exportProfile)
-	{
-		cmsCloseProfile(rs->exportProfile);
-		rs->exportProfile = NULL;
-	}
-	if (rs->exportProfileFilename)
-	{
-		g_free(rs->exportProfileFilename);
-		rs->exportProfileFilename = NULL;
-	}
-
-	if (filename)
-	{
-		rs->exportProfile = cmsOpenProfileFromFile(filename, "r");
-		if (rs->exportProfile)
-			rs->exportProfileFilename = filename;
-		else
-			g_free(filename);
-	}
-
-	rs_cms_prepare_transforms(rs);
+	filename = rs_conf_get_cms_profile(RS_CMS_PROFILE_EXPORT);
+	rs_cms_set_profile(rs->cms, PROFILE_EXPORT, filename);
+	g_free(filename);
 	update_preview_callback(NULL, rs);
 	return;
 }
 
-void
+static void
 gui_cms_intent_combobox_changed(GtkComboBox *combobox, gpointer user_data)
 {
 	RS_BLOB *rs = (RS_BLOB *) user_data;
 	gint active = gtk_combo_box_get_active(combobox);
 	rs_conf_set_cms_intent(CONF_CMS_INTENT, &active);
-	rs->cms_intent = active;
-	rs_cms_prepare_transforms(rs);
+	rs_cms_set_intent(rs->cms, active);
 	update_preview_callback(NULL, rs);
 	return;
 }
 
-gchar *
+static gchar *
 gui_cms_choose_profile(const gchar *path)
 {
 	gchar *ret = NULL;
@@ -321,7 +469,7 @@ gui_cms_choose_profile(const gchar *path)
 	return(ret);
 }
 
-void
+static void
 gui_cms_in_profile_button_clicked(GtkButton *button, gpointer user_data)
 {
 	GtkWidget *combobox = GTK_WIDGET(user_data);
@@ -339,7 +487,7 @@ gui_cms_in_profile_button_clicked(GtkButton *button, gpointer user_data)
 	return;
 }
 
-void
+static void
 gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_data)
 {
 	GtkWidget *combobox = GTK_WIDGET(user_data);
@@ -357,7 +505,7 @@ gui_cms_di_profile_button_clicked(GtkButton *button, gpointer user_data)
 	return;
 }
 
-void
+static void
 gui_cms_ex_profile_button_clicked(GtkButton *button, gpointer user_data)
 {
 	GtkWidget *combobox = GTK_WIDGET(user_data);
@@ -536,7 +684,7 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cms_intent_combobox), _("Relative colormetric"));
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cms_intent_combobox), _("Saturation"));
 	gtk_combo_box_append_text(GTK_COMBO_BOX(cms_intent_combobox), _("Absolute colormetric"));
-	gtk_combo_box_set_active(GTK_COMBO_BOX(cms_intent_combobox), rs->cms_intent);
+	gtk_combo_box_set_active(GTK_COMBO_BOX(cms_intent_combobox), rs_cms_get_intent(rs->cms));
 	gtk_box_pack_start (GTK_BOX (cms_intent_hbox), cms_intent_label, TRUE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (cms_intent_hbox), cms_intent_combobox, FALSE, TRUE, 0);
 	gtk_box_pack_start (GTK_BOX (cms_page), cms_intent_hbox, FALSE, TRUE, 0);
@@ -560,3 +708,140 @@ gui_preferences_make_cms_page(RS_BLOB *rs)
 
 	return cms_page;
 }
+
+
+void
+spinner_set_conf(GtkSpinButton *spinner, gpointer user_data)
+{
+	const gchar *path = user_data;
+	rs_conf_set_integer(path, gtk_spin_button_get_value_as_int(spinner));
+	return;
+}
+
+GtkWidget *
+spinner_from_conf(const gchar *conf, gchar *labeltext, gint default_value)
+{
+	gint value = default_value;
+	GtkWidget *spinner;
+	GtkWidget *label;
+	GtkWidget *box;
+
+	box = gtk_hbox_new(FALSE, 4);
+
+	label = gtk_label_new(labeltext);
+
+	if (!rs_conf_get_integer(conf, &value))
+		value = default_value;
+	spinner = gtk_spin_button_new_with_range(1.0, 65535.0, 1.0);
+	gtk_spin_button_set_value(GTK_SPIN_BUTTON(spinner), value);
+	g_signal_connect ((gpointer) spinner, "value-changed",
+		G_CALLBACK (spinner_set_conf), (gpointer) conf);
+
+	gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), spinner, FALSE, TRUE, 0);
+
+	return(box);
+}
+
+void
+slider_with_spinner_set_conf(GtkSpinButton *spin, gpointer user_data)
+{
+	const gchar *path = user_data;
+	rs_conf_set_integer(path, gtk_spin_button_get_value_as_int(spin));
+	return;
+}
+
+GtkWidget *
+slider_with_spinner_from_conf(const gchar *conf, gchar *labeltext, gint default_value, 
+			gdouble min, gdouble max, gdouble step_increment, gdouble page_increment, gdouble page_size)
+{
+	GtkObject *adj;
+	GtkWidget *label;
+	GtkWidget *scale;
+	GtkWidget *spin;
+	GtkWidget *box;
+	gint conf_value;
+
+	if (!rs_conf_get_integer(conf, &conf_value))
+		conf_value = default_value;
+
+	adj = gtk_adjustment_new((gdouble) conf_value, min, max, step_increment, page_increment, page_size);
+	label = gtk_label_new(labeltext);
+	scale = gtk_hscale_new(GTK_ADJUSTMENT(adj));
+	gtk_scale_set_draw_value(GTK_SCALE(scale), FALSE);
+	spin = gtk_spin_button_new(GTK_ADJUSTMENT(adj), 1.0, 0);
+	g_signal_connect((gpointer) spin, "value-changed", G_CALLBACK(slider_with_spinner_set_conf), (gpointer) conf);
+
+	box = gtk_hbox_new(FALSE, 2);
+	gtk_box_pack_start (GTK_BOX (box), label, FALSE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), scale, TRUE, TRUE, 0);
+	gtk_box_pack_start (GTK_BOX (box), spin, FALSE, TRUE, 0);
+
+	return(box);
+}
+
+/* copied verbatim from Gimp: app/widgets/gimpdock.c */
+gboolean
+window_key_press_event (GtkWidget   *widget,
+                           GdkEventKey *event)
+{
+  GtkWindow *window  = GTK_WINDOW (widget);
+  GtkWidget *focus   = gtk_window_get_focus (window);
+  gboolean   handled = FALSE;
+
+  /* we're overriding the GtkWindow implementation here to give
+   * the focus widget precedence over unmodified accelerators
+   * before the accelerator activation scheme.
+   */
+
+  /* text widgets get all key events first */
+  if (G_UNLIKELY (GTK_IS_EDITABLE (focus) || GTK_IS_TEXT_VIEW (focus)))
+    handled = gtk_window_propagate_key_event (window, event);
+
+  /* invoke control/alt accelerators */
+  if (! handled && event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK))
+    handled = gtk_window_activate_key (window, event);
+
+  /* invoke focus widget handlers */
+  if (! handled)
+    handled = gtk_window_propagate_key_event (window, event);
+
+  /* invoke non-(control/alt) accelerators */
+  if (! handled && ! (event->state & (GDK_CONTROL_MASK | GDK_MOD1_MASK)))
+    handled = gtk_window_activate_key (window, event);
+
+  /* chain up, bypassing gtk_window_key_press(), to invoke binding set */
+  if (! handled)
+    handled = GTK_WIDGET_CLASS (g_type_class_peek (g_type_parent (GTK_TYPE_WINDOW)))->key_press_event (widget, event);
+
+  return handled;
+}
+
+/* Should be used like this:
+	gtk_menu_popup(GTK_MENU(menu), NULL, NULL, pos_menu_below_widget, widget, 0, GDK_CURRENT_TIME);
+*/
+void
+pos_menu_below_widget (GtkMenu *menu, gint *x, gint *y, gboolean *push_in, gpointer user_data)
+{
+	GtkWidget *widget = GTK_WIDGET (user_data);
+    gint origin_x, origin_y;
+
+	gdk_window_get_origin (widget->window, &origin_x, &origin_y);
+
+	*x = origin_x + widget->allocation.x;
+	*y = origin_y + widget->allocation.y + widget->allocation.height;
+	*push_in = TRUE;
+	return;
+}
+
+GtkWidget *
+gui_framed(GtkWidget *widget, const gchar *title, GtkShadowType shadowtype)
+{
+	GtkWidget *frame;
+	
+	frame = gtk_frame_new(title);
+	gtk_frame_set_shadow_type(GTK_FRAME(frame), shadowtype);
+	gtk_container_add (GTK_CONTAINER (frame), widget);
+
+	return(frame);
+}
