@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -27,15 +27,16 @@
 #include <string.h>
 #include "rawfile.h"
 
-static int cpuorder;
+#if BYTE_ORDER == LITTLE_ENDIAN
+const static gushort cpuorder = 0x4949;
+#elif BYTE_ORDER == BIG_ENDIAN
+const static gushort cpuorder = 0x4D4D;
+#endif
 
 void
-raw_init()
+raw_init(void)
 {
-	if (ntohs(0x1234) == 0x1234)
-		cpuorder = 0x4D4D;
-	else
-		cpuorder = 0x4949;
+	/* stub */
 	return;
 }
 
@@ -142,6 +143,14 @@ raw_strcpy(RAWFILE *rawfile, guint pos, void *target, gint len)
 	return(TRUE);
 }
 
+gchar *
+raw_strdup(RAWFILE *rawfile, guint pos, gint len)
+{
+	if((rawfile->base+pos+len) > rawfile->size)
+		return(FALSE);
+	return(g_strndup(rawfile->map+rawfile->base+pos, len));
+}
+
 GdkPixbuf *
 raw_get_pixbuf(RAWFILE *rawfile, guint pos, guint length)
 {
@@ -153,8 +162,6 @@ raw_get_pixbuf(RAWFILE *rawfile, guint pos, guint length)
 	pl = gdk_pixbuf_loader_new();
 	gdk_pixbuf_loader_write(pl, rawfile->map+rawfile->base+pos, length, NULL);
 	pixbuf = gdk_pixbuf_loader_get_pixbuf(pl);
-	if (pixbuf)
-		g_object_ref(pixbuf);
 	gdk_pixbuf_loader_close(pl, NULL);
 	return(pixbuf);
 }
