@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -17,5 +17,10 @@
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
  */
 
-gboolean rs_jpeg_save(RS_IMAGE8 *image, const gchar *filename,
+#ifndef RS_JPEG_H
+#define RS_JPEG_H
+
+extern gboolean rs_jpeg_save(RS_IMAGE8 *image, const gchar *filename,
 	const gint quality, const gchar *profile_filename);
+
+#endif
