@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -20,57 +20,42 @@
 #include <gtk/gtk.h>
 #include <lcms.h>
 #include "color.h"
-#include "matrix.h"
-#include "rs-batch.h"
 #include "rawstudio.h"
 #include "rs-render.h"
 
-#define DECL_RENDER(func) \
-	void func(RS_PHOTO *photo, gint width, gint height, gushort *in, \
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride, void *profile);
+static guchar previewtable8[65536];
+static gushort previewtable16[65536];
 
-DECL_RENDER(rs_render_cms_sse)
-DECL_RENDER(rs_render_cms_3dnow)
-DECL_RENDER(rs_render_cms)
-DECL_RENDER(rs_render_nocms_sse)
-DECL_RENDER(rs_render_nocms_3dnow)
-DECL_RENDER(rs_render_nocms)
-
-void rs_render_histogram_table_cmov(RS_PHOTO *photo, RS_IMAGE16 *input, guint *table);
-void rs_render_histogram_table_c(RS_PHOTO *photo, RS_IMAGE16 *input, guint *table);
-
-guchar previewtable8[65536];
-gushort previewtable16[65536];
+/* CMS or no CMS dependant function pointers - initialized by
+ * rs_render_select(bool) */
+DEFINE_RENDER(*rs_render);
+DEFINE_RENDER16(*rs_render16);
+void
+(*rs_render_pixel)(RS_PHOTO *photo, gushort *in, guchar *out, void *profile);
+void
+rs_render_pixel_cms(RS_PHOTO *photo, gushort *in, guchar *out, void *profile);
+void
+rs_render_pixel_nocms(RS_PHOTO *photo, gushort *in, guchar *out, void *profile);
 
 void
 rs_render_select(gboolean cms)
 {
-	extern gint cpuflags;
-
-	rs_render_previewtable(1.0); /* make sure previewtables are ready */
+	/* make sure previewtables are ready */
+	rs_render_previewtable(1.0);
 
 	if (cms)
 	{
-		if (cpuflags & _SSE)
-			rs_render = rs_render_cms_sse;
-		else if (cpuflags & _3DNOW)
-			rs_render = rs_render_cms_3dnow;
-		else
-			rs_render = rs_render_cms;
+		rs_render = rs_render_cms;
+		rs_render16 = rs_render16_cms;
+		rs_render_pixel = rs_render_pixel_cms;
 	}
 	else
 	{
-		if (cpuflags & _SSE)
-			rs_render = rs_render_nocms_sse;
-		else if (cpuflags & _3DNOW)
-			rs_render = rs_render_nocms_3dnow;
-		else
-			rs_render = rs_render_nocms;
+		rs_render = rs_render_nocms;
+		rs_render16 = rs_render16_nocms;
+		rs_render_pixel = rs_render_pixel_nocms;
 	}
-	if (cpuflags & _CMOV)
-		rs_render_histogram_table = rs_render_histogram_table_cmov;
-	else
-		rs_render_histogram_table = rs_render_histogram_table_c;
+
 	return;
 }
 
@@ -100,13 +85,98 @@ rs_render_previewtable(const gdouble contrast)
 	}
 }
 
-void
-rs_render_cms_sse(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride, void *profile)
+/* Function pointers - initialized by arch binders */
+DEFINE_RENDER(*rs_render_cms);
+DEFINE_RENDER16(*rs_render16_cms);
+
+/* Default Implementation */
+DEFINE_RENDER(rs_render_cms_c)
 {
-#ifdef __i386__
 	gushort *buffer = g_malloc(width*3*sizeof(gushort));
+	gint srcoffset, destoffset;
+	register gint x,y;
 	register gint r,g,b;
+	gint rr,gg,bb;
+	gint pre_mul[4];
+	for(x=0;x<4;x++)
+		pre_mul[x] = (gint) (photo->pre_mul[x]*128.0);
+	for(y=0 ; y<height ; y++)
+	{
+		destoffset = 0;
+		srcoffset = y * in_rowstride;
+		for(x=0 ; x<width ; x++)
+		{
+			rr = (in[srcoffset+R]*pre_mul[R])>>7;
+			gg = (in[srcoffset+G]*pre_mul[G])>>7;
+			bb = (in[srcoffset+B]*pre_mul[B])>>7;
+			_CLAMP65535_TRIPLET(rr,gg,bb);
+			r = (rr*photo->mati.coeff[0][0]
+				+ gg*photo->mati.coeff[0][1]
+				+ bb*photo->mati.coeff[0][2])>>MATRIX_RESOLUTION;
+			g = (rr*photo->mati.coeff[1][0]
+				+ gg*photo->mati.coeff[1][1]
+				+ bb*photo->mati.coeff[1][2])>>MATRIX_RESOLUTION;
+			b = (rr*photo->mati.coeff[2][0]
+				+ gg*photo->mati.coeff[2][1]
+				+ bb*photo->mati.coeff[2][2])>>MATRIX_RESOLUTION;
+			_CLAMP65535_TRIPLET(r,g,b);
+			buffer[destoffset++] = previewtable16[r];
+			buffer[destoffset++] = previewtable16[g];
+			buffer[destoffset++] = previewtable16[b];
+			srcoffset+=in_channels;
+		}
+		cmsDoTransform((cmsHPROFILE) profile, buffer, out+y * out_rowstride, width);
+	}
+	g_free(buffer);
+	return;
+}
+
+DEFINE_RENDER16(rs_render16_cms_c)
+{
+	gushort *buffer = g_malloc(width*3*sizeof(gushort));
+	gint srcoffset, destoffset;
+	register gint x,y;
+	register gint r,g,b;
+	gint rr,gg,bb;
+	gint pre_mul[4];
+	for(x=0;x<4;x++)
+		pre_mul[x] = (gint) (photo->pre_mul[x]*128.0);
+	for(y=0 ; y<height ; y++)
+	{
+		destoffset = 0;
+		srcoffset = y * in_rowstride;
+		for(x=0 ; x<width ; x++)
+		{
+			rr = (in[srcoffset+R]*pre_mul[R]+64)>>7;
+			gg = (in[srcoffset+G]*pre_mul[G]+64)>>7;
+			bb = (in[srcoffset+B]*pre_mul[B]+64)>>7;
+			_CLAMP65535_TRIPLET(rr,gg,bb);
+			r = (rr*photo->mati.coeff[0][0]
+				+ gg*photo->mati.coeff[0][1]
+				+ bb*photo->mati.coeff[0][2])>>MATRIX_RESOLUTION;
+			g = (rr*photo->mati.coeff[1][0]
+				+ gg*photo->mati.coeff[1][1]
+				+ bb*photo->mati.coeff[1][2])>>MATRIX_RESOLUTION;
+			b = (rr*photo->mati.coeff[2][0]
+				+ gg*photo->mati.coeff[2][1]
+				+ bb*photo->mati.coeff[2][2])>>MATRIX_RESOLUTION;
+			_CLAMP65535_TRIPLET(r,g,b);
+			buffer[destoffset++] = previewtable16[r];
+			buffer[destoffset++] = previewtable16[g];
+			buffer[destoffset++] = previewtable16[b];
+			srcoffset+=in_channels;
+		}
+		cmsDoTransform((cmsHPROFILE) profile, buffer, out+y * out_rowstride, width);
+	}
+	g_free(buffer);
+	return;
+}
+
+#if defined (__i386__) || defined (__x86_64__)
+DEFINE_RENDER(rs_render_cms_sse)
+{
+	gushort *buffer = g_malloc(width*3*sizeof(gushort));
+	register glong r,g,b;
 	gint destoffset;
 	gint col;
 	gfloat top[4] align(16) = {65535.0, 65535.0, 65535.0, 65535.0};
@@ -192,33 +262,29 @@ rs_render_cms_sse(RS_PHOTO *photo, gint width, gint height, gushort *in,
 	}
 	asm volatile("emms\n\t");
 	g_free(buffer);
-#endif
 	return;
 }
 
-void
-rs_render_cms_3dnow(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride, void *profile)
+DEFINE_RENDER(rs_render_cms_3dnow)
 {
-#ifdef __i386__
 	gushort *buffer = g_malloc(width*3*sizeof(gushort));
 	gint destoffset;
 	gint col;
-	register gint r=0,g=0,b=0;
+	register glong r=0,g=0,b=0;
 	gfloat mat[12] align(8);
 	gfloat top[2] align(8);
 	mat[0] = photo->mat.coeff[0][0];
 	mat[1] = photo->mat.coeff[0][1];
 	mat[2] = photo->mat.coeff[0][2];
-	mat[3] = photo->mat.coeff[0][1]*0.0;
+	mat[3] = 0.0;
 	mat[4] = photo->mat.coeff[1][0];
 	mat[5] = photo->mat.coeff[1][1];
 	mat[6] = photo->mat.coeff[1][2];
-	mat[7] = photo->mat.coeff[1][1]*0.0;
+	mat[7] = 0.0;
 	mat[8] = photo->mat.coeff[2][0];
 	mat[9] = photo->mat.coeff[2][1];
 	mat[10] = photo->mat.coeff[2][2];
-	mat[11] = photo->mat.coeff[2][1]*0.0;
+	mat[11] = 0.0;
 	top[0] = 65535.0;
 	top[1] = 65535.0;
 	asm volatile (
@@ -300,15 +366,16 @@ rs_render_cms_3dnow(RS_PHOTO *photo, gint width, gint height, gushort *in,
 	}
 	asm volatile ("femms\n\t");
 	g_free(buffer);
-#endif
 	return;
 }
+#endif
 
-void
-rs_render_cms(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride, void *profile)
+/* No CMS renderer pointer functions - initialized by arch binder */
+DEFINE_RENDER(*rs_render_nocms);
+DEFINE_RENDER16(*rs_render16_nocms);
+
+DEFINE_RENDER(rs_render_nocms_c)
 {
-	gushort *buffer = g_malloc(width*3*sizeof(gushort));
 	gint srcoffset, destoffset;
 	register gint x,y;
 	register gint r,g,b;
@@ -318,6 +385,46 @@ rs_render_cms(RS_PHOTO *photo, gint width, gint height, gushort *in,
 		pre_mul[x] = (gint) (photo->pre_mul[x]*128.0);
 	for(y=0 ; y<height ; y++)
 	{
+		guchar *d = out + y * out_rowstride;
+		destoffset = 0;
+		srcoffset = y * in_rowstride;
+		for(x=0 ; x<width ; x++)
+		{
+			rr = (in[srcoffset+R]*pre_mul[R]+64)>>7;
+			gg = (in[srcoffset+G]*pre_mul[G]+64)>>7;
+			bb = (in[srcoffset+B]*pre_mul[B]+64)>>7;
+			_CLAMP65535_TRIPLET(rr,gg,bb);
+			r = (rr*photo->mati.coeff[0][0]
+				+ gg*photo->mati.coeff[0][1]
+				+ bb*photo->mati.coeff[0][2])>>MATRIX_RESOLUTION;
+			g = (rr*photo->mati.coeff[1][0]
+				+ gg*photo->mati.coeff[1][1]
+				+ bb*photo->mati.coeff[1][2])>>MATRIX_RESOLUTION;
+			b = (rr*photo->mati.coeff[2][0]
+				+ gg*photo->mati.coeff[2][1]
+				+ bb*photo->mati.coeff[2][2])>>MATRIX_RESOLUTION;
+			_CLAMP65535_TRIPLET(r,g,b);
+			d[destoffset++] = previewtable8[r];
+			d[destoffset++] = previewtable8[g];
+			d[destoffset++] = previewtable8[b];
+			srcoffset+=in_channels;
+		}
+	}
+	return;
+}
+
+DEFINE_RENDER16(rs_render16_nocms_c)
+{
+	gint srcoffset, destoffset;
+	register gint x,y;
+	register gint r,g,b;
+	gint rr,gg,bb;
+	gint pre_mul[4];
+	for(x=0;x<4;x++)
+		pre_mul[x] = (gint) (photo->pre_mul[x]*128.0);
+	for(y=0 ; y<height ; y++)
+	{
+		gushort *d = out + y * out_rowstride;
 		destoffset = 0;
 		srcoffset = y * in_rowstride;
 		for(x=0 ; x<width ; x++)
@@ -336,39 +443,35 @@ rs_render_cms(RS_PHOTO *photo, gint width, gint height, gushort *in,
 				+ gg*photo->mati.coeff[2][1]
 				+ bb*photo->mati.coeff[2][2])>>MATRIX_RESOLUTION;
 			_CLAMP65535_TRIPLET(r,g,b);
-			buffer[destoffset++] = previewtable16[r];
-			buffer[destoffset++] = previewtable16[g];
-			buffer[destoffset++] = previewtable16[b];
+			d[destoffset++] = previewtable16[r];
+			d[destoffset++] = previewtable16[g];
+			d[destoffset++] = previewtable16[b];
 			srcoffset+=in_channels;
 		}
-		cmsDoTransform((cmsHPROFILE) profile, buffer, out+y * out_rowstride, width);
 	}
-	g_free(buffer);
 	return;
 }
 
-void
-rs_render_nocms_sse(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride, void *profile)
+#if defined (__i386__) || defined (__x86_64__)
+DEFINE_RENDER(rs_render_nocms_sse)
 {
-#ifdef __i386__
-	register gint r,g,b;
+	register glong r,g,b;
 	gint destoffset;
 	gint col;
 	gfloat top[4] align(16) = {65535.0, 65535.0, 65535.0, 65535.0};
 	gfloat mat[12] align(16) = {
-	photo->mat.coeff[0][0],
-	photo->mat.coeff[1][0],
-	photo->mat.coeff[2][0],
-	0.0,
-	photo->mat.coeff[0][1],
-	photo->mat.coeff[1][1],
-	photo->mat.coeff[2][1],
-	0.0,
-	photo->mat.coeff[0][2],
-	photo->mat.coeff[1][2],
-	photo->mat.coeff[2][2],
-	0.0 };
+		photo->mat.coeff[0][0],
+		photo->mat.coeff[1][0],
+		photo->mat.coeff[2][0],
+		0.0,
+		photo->mat.coeff[0][1],
+		photo->mat.coeff[1][1],
+		photo->mat.coeff[2][1],
+		0.0,
+		photo->mat.coeff[0][2],
+		photo->mat.coeff[1][2],
+		photo->mat.coeff[2][2],
+		0.0 };
 	asm volatile (
 		"movups (%2), %%xmm2\n\t" /* rs->pre_mul */
 		"movaps (%0), %%xmm3\n\t" /* matrix */
@@ -437,32 +540,28 @@ rs_render_nocms_sse(RS_PHOTO *photo, gint width, gint height, gushort *in,
 		}
 	}
 	asm volatile("emms\n\t");
-#endif
 	return;
 }
 
-void
-rs_render_nocms_3dnow(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride, void *profile)
+DEFINE_RENDER(rs_render_nocms_3dnow)
 {
-#ifdef __i386__
 	gint destoffset;
 	gint col;
-	register gint r=0,g=0,b=0;
+	register glong r=0,g=0,b=0;
 	gfloat mat[12] align(8);
 	gfloat top[2] align(8);
 	mat[0] = photo->mat.coeff[0][0];
 	mat[1] = photo->mat.coeff[0][1];
 	mat[2] = photo->mat.coeff[0][2];
-	mat[3] = photo->mat.coeff[0][1]*0.0;
+	mat[3] = 0.0;
 	mat[4] = photo->mat.coeff[1][0];
 	mat[5] = photo->mat.coeff[1][1];
 	mat[6] = photo->mat.coeff[1][2];
-	mat[7] = photo->mat.coeff[1][1]*0.0;
+	mat[7] = 0.0;
 	mat[8] = photo->mat.coeff[2][0];
 	mat[9] = photo->mat.coeff[2][1];
 	mat[10] = photo->mat.coeff[2][2];
-	mat[11] = photo->mat.coeff[2][1]*0.0;
+	mat[11] = 0.0;
 	top[0] = 65535.0;
 	top[1] = 65535.0;
 	asm volatile (
@@ -543,94 +642,97 @@ rs_render_nocms_3dnow(RS_PHOTO *photo, gint width, gint height, gushort *in,
 		}
 	}
 	asm volatile ("femms\n\t");
-#endif
+
 	return;
 }
+
+#endif /* __i386__ || __x86_64__ */
+
 void
-rs_render_nocms(RS_PHOTO *photo, gint width, gint height, gushort *in,
-	gint in_rowstride, gint in_channels, guchar *out, gint out_rowstride, void *profile)
+rs_render_pixel_cms(RS_PHOTO *photo, gushort *in, guchar *out, void *profile)
 {
-	gint srcoffset, destoffset;
-	register gint x,y;
-	register gint r,g,b;
-	gint rr,gg,bb;
-	gint pre_mul[4];
-	for(x=0;x<4;x++)
-		pre_mul[x] = (gint) (photo->pre_mul[x]*128.0);
-	for(y=0 ; y<height ; y++)
-	{
-		destoffset = 0;
-		srcoffset = y * in_rowstride;
-		guchar *d = out + y * out_rowstride;
-		for(x=0 ; x<width ; x++)
-		{
-			rr = (in[srcoffset+R]*pre_mul[R])>>7;
-			gg = (in[srcoffset+G]*pre_mul[G])>>7;
-			bb = (in[srcoffset+B]*pre_mul[B])>>7;
-			_CLAMP65535_TRIPLET(rr,gg,bb);
-			r = (rr*photo->mati.coeff[0][0]
-				+ gg*photo->mati.coeff[0][1]
-				+ bb*photo->mati.coeff[0][2])>>MATRIX_RESOLUTION;
-			g = (rr*photo->mati.coeff[1][0]
-				+ gg*photo->mati.coeff[1][1]
-				+ bb*photo->mati.coeff[1][2])>>MATRIX_RESOLUTION;
-			b = (rr*photo->mati.coeff[2][0]
-				+ gg*photo->mati.coeff[2][1]
-				+ bb*photo->mati.coeff[2][2])>>MATRIX_RESOLUTION;
-			_CLAMP65535_TRIPLET(r,g,b);
-			d[destoffset++] = previewtable8[r];
-			d[destoffset++] = previewtable8[g];
-			d[destoffset++] = previewtable8[b];
-			srcoffset+=in_channels;
-		}
-	}
+	gushort buffer[3];
+	gfloat rr, gg, bb;
+	gint r,g,b;
+
+	rr = ((gfloat) in[R]) * photo->pre_mul[R];
+	gg = ((gfloat) in[G]) * photo->pre_mul[G];
+	bb = ((gfloat) in[B]) * photo->pre_mul[B];
+
+	if (rr>65535.0)
+		rr = 65535.0;
+	else if (rr<0.0)
+		rr = 0.0;
+	if (gg>65535.0)
+		gg = 65535.0;
+	else if (gg<0.0)
+		gg = 0.0;
+	if (bb>65535.0)
+		bb = 65535.0;
+	else if (bb<0.0)
+		bb = 0.0;
+
+	r = rr*photo->mat.coeff[0][0]
+		+ gg*photo->mat.coeff[0][1]
+		+ bb*photo->mat.coeff[0][2];
+	g = rr*photo->mat.coeff[1][0]
+		+ gg*photo->mat.coeff[1][1]
+		+ bb*photo->mat.coeff[1][2];
+	b = rr*photo->mat.coeff[2][0]
+		+ gg*photo->mat.coeff[2][1]
+		+ bb*photo->mat.coeff[2][2];
+	_CLAMP65535_TRIPLET(r,g,b);
+	buffer[R] = previewtable16[r];
+	buffer[G] = previewtable16[g];
+	buffer[B] = previewtable16[b];
+	cmsDoTransform((cmsHPROFILE) profile, buffer, out, 1);
 	return;
 }
 
 void
-rs_render_histogram_table_cmov(RS_PHOTO *photo, RS_IMAGE16 *input, guint *table)
+rs_render_pixel_nocms(RS_PHOTO *photo, gushort *in, guchar *out, void *profile)
 {
-#ifdef __i386__
-	gint y,x;
-	gint srcoffset;
-	gint r,g,b,rr,gg,bb;
-	gushort *in;
-	gint pre_mul[4];
+	gfloat rr, gg, bb;
+	gint r,g,b;
 
-	if (unlikely(input==NULL)) return;
+	rr = ((gfloat) in[R]) * photo->pre_mul[R];
+	gg = ((gfloat) in[G]) * photo->pre_mul[G];
+	bb = ((gfloat) in[B]) * photo->pre_mul[B];
 
-	for(x=0;x<4;x++)
-		pre_mul[x] = (gint) (photo->pre_mul[x]*128.0);
-	in	= input->pixels;
-	for(y=0 ; y<input->h ; y++)
-	{
-		srcoffset = y * input->rowstride;
-		for(x=0 ; x<input->w ; x++)
-		{
-			rr = (in[srcoffset+R]*pre_mul[R])>>7;
-			gg = (in[srcoffset+G]*pre_mul[G])>>7;
-			bb = (in[srcoffset+B]*pre_mul[B])>>7;
-			_CLAMP65535_TRIPLET_CMOV(rr,gg,bb);
-			r = (rr*photo->mati.coeff[0][0]
-				+ gg*photo->mati.coeff[0][1]
-				+ bb*photo->mati.coeff[0][2])>>MATRIX_RESOLUTION;
-			g = (rr*photo->mati.coeff[1][0]
-				+ gg*photo->mati.coeff[1][1]
-				+ bb*photo->mati.coeff[1][2])>>MATRIX_RESOLUTION;
-			b = (rr*photo->mati.coeff[2][0]
-				+ gg*photo->mati.coeff[2][1]
-				+ bb*photo->mati.coeff[2][2])>>MATRIX_RESOLUTION;
-			_CLAMP65535_TRIPLET_CMOV(r,g,b);
-			table[previewtable8[r]]++;
-			table[256+previewtable8[g]]++;
-			table[512+previewtable8[b]]++;
-			srcoffset+=input->pixelsize;
-		}
-	}
-#endif
+	if (rr>65535.0)
+		rr = 65535.0;
+	else if (rr<0.0)
+		rr = 0.0;
+	if (gg>65535.0)
+		gg = 65535.0;
+	else if (gg<0.0)
+		gg = 0.0;
+	if (bb>65535.0)
+		bb = 65535.0;
+	else if (bb<0.0)
+		bb = 0.0;
+
+	r = rr*photo->mat.coeff[0][0]
+		+ gg*photo->mat.coeff[0][1]
+		+ bb*photo->mat.coeff[0][2];
+	g = rr*photo->mat.coeff[1][0]
+		+ gg*photo->mat.coeff[1][1]
+		+ bb*photo->mat.coeff[1][2];
+	b = rr*photo->mat.coeff[2][0]
+		+ gg*photo->mat.coeff[2][1]
+		+ bb*photo->mat.coeff[2][2];
+	_CLAMP65535_TRIPLET(r,g,b);
+	out[R] = previewtable8[r];
+	out[G] = previewtable8[g];
+	out[B] = previewtable8[b];
 	return;
 }
 
+/* Function pointer - initialiazed by arch binders */
+void
+(*rs_render_histogram_table)(RS_PHOTO *photo, RS_IMAGE16 *input, guint *table);
+
+/* Default implementation */
 void
 rs_render_histogram_table_c(RS_PHOTO *photo, RS_IMAGE16 *input, guint *table)
 {
@@ -672,3 +774,47 @@ rs_render_histogram_table_c(RS_PHOTO *photo, RS_IMAGE16 *input, guint *table)
 	}
 	return;
 }
+
+#if defined (__i386__) || defined (__x86_64__)
+void
+rs_render_histogram_table_cmov(RS_PHOTO *photo, RS_IMAGE16 *input, guint *table)
+{
+	gint y,x;
+	gint srcoffset;
+	glong r,g,b,rr,gg,bb;
+	gushort *in;
+	gint pre_mul[4];
+
+	if (unlikely(input==NULL)) return;
+
+	for(x=0;x<4;x++)
+		pre_mul[x] = (gint) (photo->pre_mul[x]*128.0);
+	in	= input->pixels;
+	for(y=0 ; y<input->h ; y++)
+	{
+		srcoffset = y * input->rowstride;
+		for(x=0 ; x<input->w ; x++)
+		{
+			rr = (in[srcoffset+R]*pre_mul[R])>>7;
+			gg = (in[srcoffset+G]*pre_mul[G])>>7;
+			bb = (in[srcoffset+B]*pre_mul[B])>>7;
+			_CLAMP65535_TRIPLET_CMOV(rr,gg,bb);
+			r = (rr*photo->mati.coeff[0][0]
+				+ gg*photo->mati.coeff[0][1]
+				+ bb*photo->mati.coeff[0][2])>>MATRIX_RESOLUTION;
+			g = (rr*photo->mati.coeff[1][0]
+				+ gg*photo->mati.coeff[1][1]
+				+ bb*photo->mati.coeff[1][2])>>MATRIX_RESOLUTION;
+			b = (rr*photo->mati.coeff[2][0]
+				+ gg*photo->mati.coeff[2][1]
+				+ bb*photo->mati.coeff[2][2])>>MATRIX_RESOLUTION;
+			_CLAMP65535_TRIPLET_CMOV(r,g,b);
+			table[previewtable8[r]]++;
+			table[256+previewtable8[g]]++;
+			table[512+previewtable8[b]]++;
+			srcoffset+=input->pixelsize;
+		}
+	}
+	return;
+}
+#endif /* (__i386__) || defined (__x86_64__) */
