@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -19,15 +19,19 @@
 
 #include <gtk/gtk.h>
 #include <math.h>
-#include "matrix.h"
-#include "rs-batch.h"
 #include "rawstudio.h"
 #include "rawfile.h"
 #include "tiff-meta.h"
+#include "adobe-coeff.h"
 
-void raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
+/* It is required having some arbitrary maximum exposure time to prevent borked
+ * shutter speed values being interpreted from the tiff.
+ * 8h seems to be reasonable, even for astronomists with extra battery packs */
+#define EXPO_TIME_MAXVAL (8*60.0*60.0)
 
-void
+static void raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta);
+
+static void
 raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 {
 	static const guchar xlat[2][256] = {
@@ -69,6 +73,7 @@ raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 	gushort fieldtag=0;
 	gushort fieldtype;
 	gushort ushort_temp1=0;
+	gfloat float_temp1=0.0, float_temp2=0.0;
 	guint valuecount;
 	guint uint_temp1=0;
 	guchar char_tmp='\0';
@@ -77,13 +82,21 @@ raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 	gint serial = 0;
 	guint ver97 = 0;
 	guchar buf97[324], ci, cj, ck;
+	gboolean magic; /* Nikon's makernote type */
 
-	if (!raw_strcmp(rawfile, offset, "Nikon", 5))
-		return;
+	if (raw_strcmp(rawfile, offset, "Nikon", 5))
+	{
+		base = offset +=10;
+		raw_get_uint(rawfile, offset+4, &tmp);
+		offset += tmp;
+		magic = TRUE;
+	}
+	else
+	{
+		magic = FALSE;
+		base = offset;
+	}
 
-	base = offset +=10;
-	raw_get_uint(rawfile, offset+4, &tmp);
-	offset += tmp;
 	if(!raw_get_ushort(rawfile, offset, &number_of_entries))
 		return;
 	if (number_of_entries>5000)
@@ -100,7 +113,7 @@ raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 		offset += 8;
 
 		save = offset + 4;
-		if (valuecount * ("1112481124848"[fieldtype < 13 ? fieldtype:0]-'0') > 4)
+		if ((valuecount * ("1112481124848"[fieldtype < 13 ? fieldtype:0]-'0') > 4) && magic)
 		{
 			raw_get_uint(rawfile, offset, &uint_temp1);
 			offset = base + uint_temp1;
@@ -110,6 +123,32 @@ raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 			case 0x0002: /* ISO */
 				raw_get_ushort(rawfile, offset+2, &meta->iso);
 				break;
+			case 0x000c: /* D1 White Balance */
+				raw_get_uint(rawfile, offset, &uint_temp1);
+
+				/* This is fucked, where did these two magic constants come from? */
+				raw_get_float(rawfile, uint_temp1, &float_temp1);
+				raw_get_float(rawfile, uint_temp1+4, &float_temp2);
+				meta->cam_mul[0] = (gdouble) (float_temp1/float_temp2)/2.218750;
+
+				raw_get_float(rawfile, uint_temp1+8, &float_temp1);
+				raw_get_float(rawfile, uint_temp1+12, &float_temp2);
+				meta->cam_mul[2] = (gdouble) (float_temp1/float_temp2)/1.148438;
+
+				raw_get_float(rawfile, uint_temp1+16, &float_temp1);
+				raw_get_float(rawfile, uint_temp1+20, &float_temp2);
+				meta->cam_mul[1] = (gdouble) (float_temp1/float_temp2);
+
+				raw_get_float(rawfile, uint_temp1+24, &float_temp1);
+				raw_get_float(rawfile, uint_temp1+28, &float_temp2);
+				meta->cam_mul[3] = (gdouble) (float_temp1/float_temp2);
+				rs_metadata_normalize_wb(meta);
+				break;
+			case 0x0011: /* NikonPreview */
+				raw_get_uint(rawfile, offset, &uint_temp1);
+				raw_ifd_walker(rawfile, uint_temp1+base, meta);
+				meta->thumbnail_start += base;
+				break;
 			case 0x0097: /* white balance */
 				for(i=0;i<4;i++)
 				{
@@ -173,13 +212,13 @@ raw_nikon_makernote(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 			case 0x00a7: /* white balance */
 				if (ver97 >> 8 == 2)
 				{
-					guchar tmp[4];
-					raw_get_uchar(rawfile, offset++, tmp);
-					raw_get_uchar(rawfile, offset++, tmp+1);
-					raw_get_uchar(rawfile, offset++, tmp+2);
-					raw_get_uchar(rawfile, offset, tmp+3);
+					guchar ctmp[4];
+					raw_get_uchar(rawfile, offset++, ctmp);
+					raw_get_uchar(rawfile, offset++, ctmp+1);
+					raw_get_uchar(rawfile, offset++, ctmp+2);
+					raw_get_uchar(rawfile, offset, ctmp+3);
 					ci = xlat[0][serial & 0xff];
-					cj = xlat[1][tmp[0]^tmp[1]^tmp[2]^tmp[3]];
+					cj = xlat[1][ctmp[0]^ctmp[1]^ctmp[2]^ctmp[3]];
 					ck = 0x60;
 					for (i=0; i < 324; i++)
 						buf97[i] ^= (cj += ci * ck++);
@@ -305,8 +344,15 @@ raw_ifd_walker(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 					raw_get_short(rawfile, uint_temp1+2, &meta->focallength);
 				}
 				break;
+			case 0x0110: /* Model */
+				raw_get_uint(rawfile, offset, &uint_temp1);
+				if (!meta->model_ascii)
+					meta->model_ascii = raw_strdup(rawfile, uint_temp1, 32);
+				break;
 			case 0x010f: /* Make */
 				raw_get_uint(rawfile, offset, &uint_temp1);
+				if (!meta->make_ascii)
+					meta->make_ascii = raw_strdup(rawfile, uint_temp1, 32);
 				if (raw_strcmp(rawfile, uint_temp1, "Canon", 5))
 					meta->make = MAKE_CANON;
 				else if (raw_strcmp(rawfile, uint_temp1, "NIKON", 5))
@@ -320,6 +366,14 @@ raw_ifd_walker(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 					meta->preview_start += rawfile->base;
 				}
 				break;
+			case 0x0081: /* Minolta DiMAGE 5 */
+				if (meta->make == MAKE_MINOLTA)
+				{
+					raw_get_uint(rawfile, offset, &meta->thumbnail_start);
+					meta->thumbnail_start += rawfile->base;
+					meta->thumbnail_length = valuecount;
+				}
+				break;
 			case 0x0089: /* Minolta */
 			case 0x0117: /* PreviewImageLength */
 				if (meta->preview_length==0)
@@ -342,6 +396,14 @@ raw_ifd_walker(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 			case 0x0202: /* jpeg length */
 				raw_get_uint(rawfile, offset, &meta->thumbnail_length);
 				break;
+			case 0x829A: /* Exposure time */
+				raw_get_uint(rawfile, offset, &uint_temp1);
+				raw_get_float(rawfile, uint_temp1, &float_temp1);
+				raw_get_float(rawfile, uint_temp1+4, &float_temp2);
+				float_temp1 /= float_temp2;
+				if (float_temp1 < EXPO_TIME_MAXVAL)
+					meta->shutterspeed = 1/float_temp1;
+				break;
 			case 0x829D: /* FNumber */
 				raw_get_uint(rawfile, offset, &uint_temp1);
 				raw_get_float(rawfile, uint_temp1, &float_temp1);
@@ -373,7 +435,9 @@ raw_ifd_walker(RAWFILE *rawfile, guint offset, RS_METADATA *meta)
 				raw_get_uint(rawfile, offset, &uint_temp1);
 				raw_get_float(rawfile, uint_temp1, &float_temp1);
 				raw_get_float(rawfile, uint_temp1+4, &float_temp2);
-				meta->shutterspeed = 1.0/pow(2.0,-float_temp1/float_temp2);
+				float_temp1 /= -float_temp2;
+				if (float_temp1 < EXPO_TIME_MAXVAL)
+					meta->shutterspeed = 1.0/pow(2.0, float_temp1);
 				break;
 			case 0x4001: /* white balance for Canon 20D & 350D */
 				raw_get_uint(rawfile, offset, &uint_temp1);
@@ -427,6 +491,7 @@ rs_tiff_load_meta(const gchar *filename, RS_METADATA *meta)
 	meta->cam_mul[1] = 1.0;
 	meta->cam_mul[2] = 1.0;
 	meta->cam_mul[3] = 1.0;
+	meta->model_ascii = NULL;
 
 	if(!(rawfile = raw_open_file(filename)))
 		return;
@@ -441,6 +506,8 @@ rs_tiff_load_meta(const gchar *filename, RS_METADATA *meta)
 		offset = next;
 	} while (next>0);
 
+	adobe_coeff_set(&meta->adobe_coeff, NULL, meta->model_ascii);
+
 	raw_close_file(rawfile);
 }
 
@@ -498,7 +565,12 @@ rs_tiff_load_thumb(const gchar *src)
 	{
 		gdouble ratio;
 
-		pixbuf = raw_get_pixbuf(rawfile, start, length);
+		if ((length==165888) && (meta->make == MAKE_CANON))
+			pixbuf = gdk_pixbuf_new_from_data(rawfile->map+start, GDK_COLORSPACE_RGB, FALSE, 8, 288, 192, 288*3, NULL, NULL);
+		else if ((length==57600) && (meta->make == MAKE_NIKON))
+			pixbuf = gdk_pixbuf_new_from_data(rawfile->map+start, GDK_COLORSPACE_RGB, FALSE, 8, 160, 120, 160*3, NULL, NULL);
+		else
+			pixbuf = raw_get_pixbuf(rawfile, start, length);
 		if (pixbuf)
 		{
 			if ((gdk_pixbuf_get_width(pixbuf) == 160) && (gdk_pixbuf_get_height(pixbuf)==120))
