@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2006 Anders Brander <anders@brander.dk> and 
+ * Copyright (C) 2006, 2007 Anders Brander <anders@brander.dk> and 
  * Anders Kvist <akv@lnxbx.dk>
  *
  * This program is free software; you can redistribute it and/or
@@ -19,12 +19,9 @@
 
 #include <gtk/gtk.h>
 #include <string.h> /* memset() */
-#include "matrix.h"
-#include "rs-batch.h"
 #include "rawstudio.h"
 #include "gtk-helper.h"
 #include "gtk-interface.h"
-#include "color.h"
 #include "toolbox.h"
 #include "conf_interface.h"
 #include "gettext.h"
@@ -36,27 +33,26 @@ struct reset_carrier {
 };
 
 GtkLabel *infolabel;
+static GtkWidget *scale;
+static GtkWidget *toolbox;
 
-GtkWidget *gui_hist(RS_BLOB *rs, const gchar *label);
-GtkWidget *gui_box(const gchar *title, GtkWidget *in);
-void gui_transform_rot90_clicked(GtkWidget *w, RS_BLOB *rs);
-void gui_transform_rot180_clicked(GtkWidget *w, RS_BLOB *rs);
-void gui_transform_rot270_clicked(GtkWidget *w, RS_BLOB *rs);
-void gui_transform_mirror_clicked(GtkWidget *w, RS_BLOB *rs);
-void gui_transform_flip_clicked(GtkWidget *w, RS_BLOB *rs);
-GtkWidget *gui_transform(RS_BLOB *rs);
-GtkWidget *gui_tool_warmth(RS_BLOB *rs, gint n);
-GtkWidget *gui_slider(GtkObject *adj, const gchar *label);
-gboolean gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct reset_carrier *rc);
-GtkWidget *gui_make_scale_from_adj(RS_BLOB *rs, GCallback cb, GtkObject *adj, gint mask);
-GtkWidget *gui_tool_exposure(RS_BLOB *rs, gint n);
-GtkWidget *gui_tool_saturation(RS_BLOB *rs, gint n);
-GtkWidget *gui_tool_hue(RS_BLOB *rs, gint n);
-GtkWidget *gui_tool_contrast(RS_BLOB *rs, gint n);
-GtkWidget *gui_make_tools(RS_BLOB *rs, gint n);
-void gui_notebook_callback(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, RS_BLOB *rs);
+static GtkWidget *gui_hist(RS_BLOB *rs, const gchar *label);
+static GtkWidget *gui_box(const gchar *title, GtkWidget *in, gboolean expanded);
+static void gui_transform_rot90_clicked(GtkWidget *w, RS_BLOB *rs);
+static void gui_transform_rot180_clicked(GtkWidget *w, RS_BLOB *rs);
+static void gui_transform_rot270_clicked(GtkWidget *w, RS_BLOB *rs);
+static void gui_transform_mirror_clicked(GtkWidget *w, RS_BLOB *rs);
+static void gui_transform_flip_clicked(GtkWidget *w, RS_BLOB *rs);
+static GtkWidget *gui_transform(RS_BLOB *rs);
+static GtkWidget *gui_tool_warmth(RS_BLOB *rs, gint n);
+static GtkWidget *gui_slider(GtkObject *adj, const gchar *label, gboolean expanded);
+static gboolean gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct reset_carrier *rc);
+static GtkWidget *gui_make_scale_from_adj(RS_BLOB *rs, GCallback cb, GtkObject *adj, gint mask);
+static GtkWidget *gui_make_tools(RS_BLOB *rs, gint n);
+static void gui_notebook_callback(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, RS_BLOB *rs);
+static void scale_expand_callback(GObject *object, GParamSpec *param_spec, gpointer user_data);
 
-GtkWidget *
+static GtkWidget *
 gui_hist(RS_BLOB *rs, const gchar *label)
 {
 	GdkPixbuf *pixbuf;
@@ -79,17 +75,17 @@ gui_hist(RS_BLOB *rs, const gchar *label)
 	/* creates an image from the histogram pixbuf */
 	rs->histogram_image = (GtkImage *) gtk_image_new_from_pixbuf(pixbuf);
 
-	return(gui_box(label, (GtkWidget *)rs->histogram_image));
+	return(gui_box(label, (GtkWidget *)rs->histogram_image, TRUE));
 }
 
-GtkWidget *
-gui_box(const gchar *title, GtkWidget *in)
+static GtkWidget *
+gui_box(const gchar *title, GtkWidget *in, gboolean expanded)
 {
 	GtkWidget *expander, *label;
 
 	expander = gtk_expander_new (NULL);
 	gtk_widget_show (expander);
-	gtk_expander_set_expanded (GTK_EXPANDER (expander), TRUE);
+	gtk_expander_set_expanded (GTK_EXPANDER (expander), expanded);
 
 	label = gtk_label_new (title);
 	gtk_widget_show (label);
@@ -99,42 +95,47 @@ gui_box(const gchar *title, GtkWidget *in)
 	return(expander);
 }
 
-void
+static void
 gui_transform_rot90_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	ORIENTATION_90(rs->photo->orientation);
-	update_preview(rs, FALSE);
+	if (!rs->photo) return;
+	rs_photo_rotate(rs->photo, 1, 0.0);
+	update_preview(rs, FALSE, TRUE);
 }
 
-void
+static void
 gui_transform_rot180_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	ORIENTATION_180(rs->photo->orientation);
-	update_preview(rs, FALSE);
+	if (!rs->photo) return;
+	rs_photo_rotate(rs->photo, 2, 0.0);
+	update_preview(rs, FALSE, TRUE);
 }
 
-void
+static void
 gui_transform_rot270_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	ORIENTATION_270(rs->photo->orientation);
-	update_preview(rs, FALSE);
+	if (!rs->photo) return;
+	rs_photo_rotate(rs->photo, 3, 0.0);
+	update_preview(rs, FALSE, TRUE);
 }
 
-void
+static void
 gui_transform_mirror_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	ORIENTATION_MIRROR(rs->photo->orientation);
-	update_preview(rs, FALSE);
+	if (!rs->photo) return;
+	rs_photo_mirror(rs->photo);
+	update_preview(rs, FALSE, TRUE);
 }
 
-void
+static void
 gui_transform_flip_clicked(GtkWidget *w, RS_BLOB *rs)
 {
-	ORIENTATION_FLIP(rs->photo->orientation);
-	update_preview(rs, FALSE);
+	if (!rs->photo) return;
+	rs_photo_flip(rs->photo);
+	update_preview(rs, FALSE, TRUE);
 }
 
-GtkWidget *
+static GtkWidget *
 gui_transform(RS_BLOB *rs)
 {
 	GtkWidget *hbox;
@@ -170,10 +171,10 @@ gui_transform(RS_BLOB *rs)
 	gtk_box_pack_start(GTK_BOX (hbox), rot270, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX (hbox), rot180, FALSE, FALSE, 0);
 	gtk_box_pack_start(GTK_BOX (hbox), rot90, FALSE, FALSE, 0);
-	return(gui_box(_("Transforms"), hbox));
+	return(gui_box(_("Transforms"), hbox, TRUE));
 }
 
-GtkWidget *
+static GtkWidget *
 gui_tool_warmth(RS_BLOB *rs, gint n)
 {
 	GtkWidget *box;
@@ -186,27 +187,27 @@ gui_tool_warmth(RS_BLOB *rs, gint n)
 	box = gtk_vbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), wscale, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (box), tscale, FALSE, FALSE, 0);
-	return(gui_box(_("Warmth/tint"), box));
+	return(gui_box(_("Warmth/tint"), box, TRUE));
 }
 
-GtkWidget *
-gui_slider(GtkObject *adj, const gchar *label)
+static GtkWidget *
+gui_slider(GtkObject *adj, const gchar *label, gboolean expanded)
 {
 	GtkWidget *hscale;
 	hscale = gtk_hscale_new (GTK_ADJUSTMENT (adj));
 	gtk_scale_set_value_pos( GTK_SCALE(hscale), GTK_POS_LEFT);
 	gtk_scale_set_digits(GTK_SCALE(hscale), 2);
-	return(gui_box(label, hscale));
+	return(gui_box(label, hscale, expanded));
 }
 
-gboolean
+static gboolean
 gui_adj_reset_callback(GtkWidget *widget, GdkEventButton *event, struct reset_carrier *rc)
 {
 	rs_settings_reset(rc->rs->settings[rc->rs->current_setting], rc->mask);
 	return(TRUE);
 }
 
-GtkWidget *
+static GtkWidget *
 gui_make_scale_from_adj(RS_BLOB *rs, GCallback cb, GtkObject *adj, gint mask)
 {
 	GtkWidget *hscale, *box, *rimage, *revent;
@@ -237,72 +238,85 @@ gui_make_scale_from_adj(RS_BLOB *rs, GCallback cb, GtkObject *adj, gint mask)
 	return(box);
 }
 
-GtkWidget *
-gui_tool_exposure(RS_BLOB *rs, gint n)
-{
-	GtkWidget *hscale;
-
-	hscale = gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback), rs->settings[n]->exposure, MASK_EXPOSURE);
-	return(gui_box(_("Exposure"), hscale));
-}
-
-GtkWidget *
-gui_tool_saturation(RS_BLOB *rs, gint n)
-{
-	GtkWidget *hscale;
-
-	hscale = gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback), rs->settings[n]->saturation, MASK_SATURATION);
-	return(gui_box(_("Saturation"), hscale));
-}
-
-GtkWidget *
-gui_tool_hue(RS_BLOB *rs, gint n)
-{
-	GtkWidget *hscale;
-
-	hscale = gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback), rs->settings[n]->hue, MASK_HUE);
-	return(gui_box(_("Hue"), hscale));
-}
-
-GtkWidget *
-gui_tool_contrast(RS_BLOB *rs, gint n)
-{
-	GtkWidget *hscale;
-
-	hscale = gui_make_scale_from_adj(rs, G_CALLBACK(update_previewtable_callback), rs->settings[n]->contrast, MASK_CONTRAST);
-	return(gui_box(_("Contrast"), hscale));
-}
-
-GtkWidget *
+static GtkWidget *
 gui_make_tools(RS_BLOB *rs, gint n)
 {
-	GtkWidget *toolbox;
+	GtkWidget *tbox;
 
-	toolbox = gtk_vbox_new (FALSE, 0);
-	gtk_widget_show (toolbox);
-	gtk_box_pack_start (GTK_BOX (toolbox), gui_tool_exposure(rs, n), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (toolbox), gui_tool_saturation(rs, n), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (toolbox), gui_tool_hue(rs, n), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (toolbox), gui_tool_contrast(rs, n), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (toolbox), gui_tool_warmth(rs, n), FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (toolbox), gui_transform(rs), FALSE, FALSE, 0);
-	return(toolbox);
+	tbox = gtk_vbox_new (FALSE, 0);
+	gtk_widget_show (tbox);
+	gtk_box_pack_start (GTK_BOX (tbox), gui_box(_("Exposure"),
+		gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback),
+		rs->settings[n]->exposure, MASK_EXPOSURE), TRUE), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (tbox), gui_box(_("Saturation"),
+		gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback),
+		rs->settings[n]->saturation, MASK_SATURATION), TRUE), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (tbox), gui_box(_("Hue"),
+		gui_make_scale_from_adj(rs, G_CALLBACK(update_preview_callback),
+		rs->settings[n]->hue, MASK_HUE), TRUE), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (tbox), gui_box(_("Contrast"),
+		gui_make_scale_from_adj(rs, G_CALLBACK(update_previewtable_callback),
+		rs->settings[n]->contrast, MASK_CONTRAST), TRUE), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (tbox), gui_tool_warmth(rs, n), FALSE, FALSE, 0);
+	return(tbox);
 }
 
-void
+static void
 gui_notebook_callback(GtkNotebook *notebook, GtkNotebookPage *page, guint page_num, RS_BLOB *rs)
 {
 	rs->current_setting = page_num;
 	if (rs->photo) {
 		rs->photo->current_setting = rs->current_setting;
-		update_preview_callback(NULL, rs);
+		update_previewtable_callback(NULL, rs);
 	}
 }
 
+static void
+scale_expand_callback(GObject *object, GParamSpec *param_spec, gpointer user_data)
+{
+	RS_BLOB *rs = (RS_BLOB *) user_data;
+
+	if(gtk_expander_get_expanded (GTK_EXPANDER (object)))
+	{
+		rs->zoom_to_fit = FALSE;
+		update_preview(rs, FALSE, TRUE);
+	}
+	else
+		rs_zoom_to_fit(rs);
+	return;
+}
+
+void
+scale_expand_set(gboolean expanded)
+{
+	gtk_expander_set_expanded (GTK_EXPANDER (scale), expanded);
+	return;
+}
+
+void
+gui_toolbox_add_widget(GtkWidget *widget)
+{
+	gtk_box_pack_start (GTK_BOX (toolbox), widget, FALSE, FALSE, 0);
+	gtk_widget_show_all(widget);
+	return;
+}
+
+GtkWidget *
+gui_toolbox_add_tool_frame(GtkWidget *widget, gchar *title)
+{
+	GtkWidget *frame;
+	frame = gtk_frame_new(title);
+
+	gtk_container_set_border_width(GTK_CONTAINER(frame), 4);
+	gtk_container_add(GTK_CONTAINER(frame), widget);
+
+	gui_toolbox_add_widget(frame);
+	return frame;
+}
+
 GtkWidget *
 make_toolbox(RS_BLOB *rs)
 {
-	GtkWidget *toolbox;
 	GtkWidget *notebook;
 	GtkWidget *label1;
 	GtkWidget *label2;
@@ -320,9 +334,13 @@ make_toolbox(RS_BLOB *rs)
 	gtk_notebook_append_page(GTK_NOTEBOOK(notebook), gui_make_tools(rs, 2), label3);
 	g_signal_connect(notebook, "switch-page", G_CALLBACK(gui_notebook_callback), rs);
 
+	scale = gui_slider(rs->scale, _("Scale"), FALSE);
+	g_signal_connect(scale, "notify::expanded", G_CALLBACK (scale_expand_callback), rs);
+
 	toolbox = gtk_vbox_new (FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (toolbox), notebook, FALSE, FALSE, 0);
-	gtk_box_pack_start (GTK_BOX (toolbox), gui_slider(rs->scale, _("Scale")), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (toolbox), gui_transform(rs), FALSE, FALSE, 0);
+	gtk_box_pack_start (GTK_BOX (toolbox), scale, FALSE, FALSE, 0);
 	gtk_box_pack_start (GTK_BOX (toolbox), gui_hist(rs, _("Histogram")), FALSE, FALSE, 0);
 
 	infolabel = (GtkLabel *) gtk_label_new_with_mnemonic("");
